//!
//!
use super::{models, schema::captchas};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of captchas").finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_captcha: &models::NewCaptcha,
  ) -> Result<models::Captcha, Error> {
    let captcha = diesel::insert_into(captchas::table)
      .values(new_captcha)
      .get_result::<models::Captcha>(conn)?;

    Ok(captcha)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::Captcha>, Error> {
    match captchas::table.find(id).first::<models::Captcha>(conn) {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: models::FindAll,
  ) -> Result<i64, Error> {
    use captchas::dsl;

    let mut q = captchas::table.into_boxed();

    if let Some(sp) = find_all.expires_at {
      q = q.filter(dsl::expires_at.lt(sp));
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: models::FindAll,
  ) -> Result<Vec<models::Captcha>, Error> {
    use captchas::dsl;

    let mut q = captchas::table.into_boxed();

    if let Some(sp) = find_all.expires_at {
      q = q.filter(dsl::expires_at.lt(sp));
    }

    if let Some(p) = find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = find_all.sorts {
      for s in orderbys {
        match s {
          bcr::models::pagination::Sort::ASC(property) => match property.as_str() {
            "expires_at" => {
              q = q.order_by(dsl::expires_at.asc());
            }
            "created_at" => {
              q = q.order_by(dsl::created_at.asc());
            }

            _ => {}
          },
          bcr::models::pagination::Sort::DESC(property) => match property.as_str() {
            "expires_at" => {
              q = q.order_by(dsl::expires_at.desc());
            }
            "created_at" => {
              q = q.order_by(dsl::created_at.desc());
            }
            _ => {}
          },
        };
      }
    }

    q.load::<models::Captcha>(conn)
  }

  ///
  pub fn delete_expired(&self, conn: &diesel::PgConnection) -> Result<u64, Error> {
    use captchas::dsl;

    let now = chrono::Utc::now().timestamp();

    diesel::delete(dsl::captchas.filter(dsl::expires_at.le(now)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
